/*******************************************************************************
  This file is part of the kde-game 'kjumpingCube'

  Copyright (C) 1998 by Matthias Kiefer <Matthias.Kiefer@stud.uni-karlsruhe.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  *******************************************************************************/  

#include <kapp.h>
#include <kmenubar.h>
#include <ktoolbar.h>
#include <kstatusbar.h>
#include <kstdaccel.h>
#include <qkeycode.h>
#include <qaccel.h>
#include <kmsgbox.h>
#include <kiconloader.h>
#include <klocale.h>
#include <qpalette.h>
#include <kconfig.h>
#include <kcolordlg.h>


#include "topwindow.h"
#include "cubebox.h"


#define DEFAULT_ZOOM 5
#define DEFAULT_DIM 8

TopWindow::TopWindow(const char *name) : KTopLevelWidget(name)
{
  int cubeDim;
  int zoom;

  // create menus

  createMenubar();
  createToolbar();
  createStatusbar();
  
  if(!kapp->isRestored())
    {
      KConfig *config=kapp->getConfig();
      if(config != 0)
	{  
	  zoom=config->readNumEntry("Zoom",DEFAULT_ZOOM);
	  cubeDim=config->readNumEntry("CubeDim",DEFAULT_DIM);
	  
	  int menuPos = config->readNumEntry("MenubarPos",(int)(KMenuBar::Top));
	  menu->setMenuBarPos((KMenuBar::menuPosition)menuPos);
	  
	  int barPos = config->readNumEntry("ToolbarPos",(int)(KToolBar::Top));
	  toolbar->setBarPos((KToolBar::BarPosition)barPos);
	  
	  bool visible=(bool)(config->readNumEntry("Toolbar",(int)(TRUE)));
	  options->setItemChecked(ID_VIEW_TOOLBAR,visible);
	  if(visible)
	    enableToolBar(KToolBar::Show);
	  else 
	    enableToolBar(KToolBar::Hide);
	  
	  
	  visible=(bool)(config->readNumEntry("Statusbar",(int)(TRUE)));
	  options->setItemChecked(ID_VIEW_STATUSBAR,visible);
	  if(visible)
	    enableStatusBar(KStatusBar::Show);
	  else 
	    enableStatusBar(KStatusBar::Hide);
	  
	  
	  color1=new QPalette(config->readColorEntry("Color1",&QColor("darkred")));
	  color2=new QPalette(config->readColorEntry("Color2",&QColor("darkblue")));
	  
	}      
      else 
	{
	  setDefaultValues(cubeDim,zoom);
	  debug("using default values");
	}
      
      createNewCubebox(cubeDim,zoom);
      
      int skill=config->readNumEntry("Skill",1);
      cubebox->setSkill(skill);
      skillMenu->setItemChecked(ID_SKILL_BASE+skill,TRUE);
      
      bool flag=(bool)config->readNumEntry("Computer Pl.1",(int)FALSE);
      cubebox->setComputerplayer(1,flag);
      game->setItemChecked(ID_COMPUTER_BASE+1,flag);
      flag=(bool)config->readNumEntry("Computer Pl.2",(int)TRUE);
      cubebox->setComputerplayer(2,flag);
      game->setItemChecked(ID_COMPUTER_BASE+2,flag);
      
      updatePlayfieldMenu(cubeDim);
      updateZoomMenu(cubebox->zoom());
            
      show();
      
      cubebox->checkComputerplayer(1);
    }
    
}


TopWindow::~TopWindow()
{
   if(cubebox)          
     delete cubebox;
  if(statusbar)
      delete statusbar;
  if(toolbar)
      delete toolbar;
  if(menu)
      delete menu;
  if(playfieldMenu)
      delete playfieldMenu;
  if(skillMenu)
     delete skillMenu;  
}



void TopWindow::createMenubar()
{
  // create KStdAccel-object
  KStdAccel *stdaccel = new KStdAccel( kapp->getConfig() );
  // don't know why kjumpingCube segmentfaults by quitting when I use:
  //KStdAccel stdaccel( kapp->getConfig() );  

  QPopupMenu *file = new QPopupMenu; 
  file->insertItem( klocale->translate("&New Game"),ID_GAME_NEW);
  file->insertSeparator();
  file->insertItem(klocale->translate("&Open Game"),ID_GAME_OPEN);
  file->insertItem(klocale->translate("&Save Game"),ID_GAME_SAVE);
  file->insertSeparator();
  file->insertItem(klocale->translate("&Quit"),ID_GAME_QUIT);
  
  file->setAccel(stdaccel->openNew(),ID_GAME_NEW);
  file->setAccel(stdaccel->open(),ID_GAME_OPEN);
  file->setAccel(stdaccel->save(),ID_GAME_SAVE);
  file->setAccel(stdaccel->quit(),ID_GAME_QUIT);

  connect(file,SIGNAL(activated(int)),SLOT(menuCallback(int)));

  zoomMenu =new QPopupMenu;
  zoomMenu->setCheckable(TRUE);
  zoomMenu->insertItem("60%",ID_ZOOM_BASE+3);
  zoomMenu->insertItem("80%",ID_ZOOM_BASE+4);
  zoomMenu->insertItem("100%",ID_ZOOM_BASE+5);
  zoomMenu->insertItem("120%",ID_ZOOM_BASE+6);
  zoomMenu->insertItem("140%",ID_ZOOM_BASE+7);
  zoomMenu->insertItem("160%",ID_ZOOM_BASE+8);

  connect(zoomMenu,SIGNAL(activated(int)),SLOT(menuCallback(int)));

 

  playfieldMenu=new QPopupMenu;
  playfieldMenu->setCheckable(TRUE);
  playfieldMenu->insertItem("&5x5",ID_FIELD +5);
  playfieldMenu->insertItem("&6x6",ID_FIELD +6);
  playfieldMenu->insertItem("&7x7",ID_FIELD +7);
  playfieldMenu->insertItem("&8x8",ID_FIELD +8);
  playfieldMenu->insertItem("&9x9",ID_FIELD +9);
  playfieldMenu->insertItem("&10x10",ID_FIELD +10);
  
  connect(playfieldMenu,SIGNAL(activated(int)),SLOT(menuCallback(int)));

  
  skillMenu=new QPopupMenu;
  skillMenu->setCheckable(TRUE);
  skillMenu->insertItem(klocale->translate("&Beginner"),ID_SKILL_BASE+1);
  skillMenu->insertItem(klocale->translate("&Novice"),ID_SKILL_BASE+2);
  skillMenu->insertItem(klocale->translate("&Average"),ID_SKILL_BASE+3);
  skillMenu->insertItem(klocale->translate("&Good"),ID_SKILL_BASE+4);
  skillMenu->insertItem(klocale->translate("&Expert"),ID_SKILL_BASE+5);

  connect(skillMenu,SIGNAL(activated(int)),SLOT(menuCallback(int)));
  
  
  options = new QPopupMenu;
  options->setCheckable(TRUE);
  options->insertItem(klocale->translate("S&kill"),skillMenu);
  options->insertSeparator();
  options->insertItem(klocale->translate("&Playfield"),playfieldMenu);
  options->insertItem(klocale->translate("Set &Zoom"),zoomMenu);
  options->insertItem(klocale->translate("Zoom &In"),ID_OPT_ZOOMIN);
  options->insertItem(klocale->translate("Zoom &Out"),ID_OPT_ZOOMOUT);
  options->insertSeparator();
  options->insertItem(klocale->translate("Color Player &1"),ID_COLOR_BASE+1);
  options->insertItem(klocale->translate("Color Player &2"),ID_COLOR_BASE+2);
  options->insertSeparator();
  options->insertItem(klocale->translate("Show &Toolbar"),ID_VIEW_TOOLBAR);
  options->insertItem(klocale->translate("Show St&atusbar"),ID_VIEW_STATUSBAR);
  //options->insertSeparator();
  //options->insertItem(klocale->translate("&Default Values"),ID_OPT_DEFAULT);
  options->insertSeparator();
  options->insertItem(klocale->translate("&Save Settings"),ID_OPT_SAVE);

  //options->setAccel(CTRL+Key_D,ID_OPT_DEFAULT);
  options->setAccel(CTRL+Key_1,ID_COLOR_BASE+1);
  options->setAccel(CTRL+Key_2,ID_COLOR_BASE+2);
  options->setAccel(CTRL+Key_Plus,ID_OPT_ZOOMIN);
  options->setAccel(CTRL+Key_Minus,ID_OPT_ZOOMOUT);
  
  connect(options,SIGNAL(activated(int)),SLOT(menuCallback(int)));

    
  game=new QPopupMenu;
  game->setCheckable(TRUE);
  game->insertItem(klocale->translate("Get &Hint"),ID_GAME_HINT);
  game->insertItem(klocale->translate("&Stop Thinking"),ID_GAME_STOP_HINT);
  game->setItemEnabled(ID_GAME_STOP_HINT,FALSE);
  game->insertItem(klocale->translate("&Continue"),ID_GAME_CONTINUE);
  game->setItemEnabled(ID_GAME_CONTINUE,FALSE);
  game->insertSeparator();
  game->insertItem(klocale->translate("Computer plays Player &1"),ID_COMPUTER_BASE+1);
  game->insertItem(klocale->translate("Computer plays Player &2"),ID_COMPUTER_BASE+2);
  game->insertSeparator();
  game->insertItem(klocale->translate("&Undo Move"),ID_GAME_UNDO);
    
  game->setItemEnabled(ID_GAME_UNDO,FALSE);
      
  connect(game,SIGNAL(activated(int)),SLOT(menuCallback(int)));
 
  game->setAccel(Key_Escape,ID_GAME_STOP_HINT);
  game->setAccel(CTRL+Key_H,ID_GAME_HINT);
  game->setAccel(stdaccel->undo(),ID_GAME_UNDO);

  
  QPopupMenu *help = new QPopupMenu;
  help->insertItem(klocale->translate("&Contents"),ID_HELP_CONTENTS);
  help->insertItem( klocale->translate("&About..."),ID_HELP_ABOUT);
  
  
  connect(help,SIGNAL(activated(int)),SLOT(menuCallback(int)));
 
  help->setAccel(stdaccel->help(),ID_HELP_CONTENTS);

  menu = new KMenuBar( this );
  CHECK_PTR( menu );
  menu->insertItem( klocale->translate("&File"), file );
  menu->insertItem(klocale->translate("&Game"),game);
  menu->insertItem(klocale->translate("&Options"),options);
  menu->insertSeparator();
  menu->insertItem( klocale->translate("&Help"), help );
  

  connect(menu,SIGNAL(moved(menuPosition)),SLOT(barPositionChanged()));
 
  
  menu->show();
  setMenu(menu);

  // delete stdaccel 

}


void TopWindow::createToolbar()
{  
  KIconLoader *loader = kapp->getIconLoader();

  toolbar = new KToolBar(this);
  CHECK_PTR( toolbar);
  
  //exitbutton
  //toolbar->insertButton(loader->loadIcon("exit.xpm"),ID_GAME_QUIT,
//			TRUE,klocale->translate("Quit"));
  
  // restartbutton
  toolbar->insertButton(loader->loadIcon("reload.xpm"),ID_GAME_NEW,
			TRUE,klocale->translate("New Game"));

  
  toolbar->insertSeparator();
  
  toolbar->insertButton(loader->loadIcon("stop.xpm"),ID_GAME_STOP_HINT,
		  TRUE,klocale->translate("Stop Thinking"));
  
  toolbar->setItemEnabled(ID_GAME_STOP_HINT,FALSE);
  
  toolbar->insertButton(loader->loadIcon("idea.xpm"),ID_GAME_HINT,
		  TRUE,klocale->translate("Get Hint"));
  
  // undo-button
  toolbar->insertButton(loader->loadIcon("back.xpm"),ID_GAME_UNDO,TRUE,
		         klocale->translate("Undo Move"));
  toolbar->setItemEnabled(ID_GAME_UNDO,FALSE);
  
  // zoominbutton
  toolbar->insertButton(loader->loadIcon("viewmag+.xpm"),ID_OPT_ZOOMIN,
			TRUE,klocale->translate("Zoom In"));
  // zoomoutButton
  toolbar->insertButton(loader->loadIcon("viewmag-.xpm"),ID_OPT_ZOOMOUT,
			TRUE,klocale->translate("Zoom Out"));

  toolbar->insertSeparator();

  //helpbutton
  toolbar->insertButton(loader->loadIcon("help.xpm"),ID_HELP_CONTENTS,TRUE,
			klocale->translate("Help"));

    
  connect(toolbar, SIGNAL(clicked(int)), this, SLOT(menuCallback(int)));
  connect(toolbar,SIGNAL(moved(BarPosition)),SLOT(barPositionChanged()));

  
  
  toolbar->show();
  addToolBar(toolbar);

}



void TopWindow::createStatusbar()
{  
  statusbar=new KStatusBar(this);
  CHECK_PTR( statusbar );
  QString s;
  s.sprintf(klocale->translate("on turn: Player %d"),1);
  
  statusbar->insertItem(s+klocale->translate("(Computer)"),ID_STATUS_TURN);
  
  /* if(cubebox->isComputer(1))
     s+=" (Computer)"; */
  statusbar->changeItem(s,ID_STATUS_TURN);
  
  statusbar->show();
  setStatusBar(statusbar);
 
}
 
void TopWindow::createNewCubebox(int dim, int size, int skill)
{
  cubebox=new CubeBox(dim , size ,color1,color2,this,"cubeBox");

  connect(cubebox,SIGNAL(playerChanged(int)),SLOT(changePlayer(int)));
  
  // have to connect slot checkComputerplayer here, because if I connect this
  // in constructor of CubeBox it will first compute next move and then
  // show the current player
  connect(cubebox,SIGNAL(playerChanged(int)),cubebox,SLOT(checkComputerplayer(int)));

  connect(cubebox,SIGNAL(stoppedThinking()),SLOT(disableStop()));	
  connect(cubebox,SIGNAL(startedThinking()),SLOT(enableStop()));	
  
  cubebox->setSkill(skill);
  
  cubebox->show();
  setView(cubebox);
  updateRects();

}

void TopWindow::setDefaultValues(int & cubeDim, int &zoom)
{
  zoom=DEFAULT_ZOOM;
  cubeDim=DEFAULT_DIM;
  // set playercolors
  if(color1)
    delete color1;
  if(color2)
    delete color2;
  QColor newColor1("darkred");
  QColor newColor2("darkblue");
  color1=new QPalette(newColor1);
  color2=new QPalette(newColor2);
}


void TopWindow::updatePlayfieldMenu(int dim)
{
  for(int i=5;i<=10;i++)
    playfieldMenu->setItemChecked(ID_FIELD +i ,FALSE);
  
  playfieldMenu->setItemChecked(ID_FIELD +dim,TRUE);
}

void TopWindow::updateZoomMenu(int zoom)
{
  for(int i=2;i<=8;i++)
    zoomMenu->setItemChecked(ID_ZOOM_BASE+i,FALSE);

  zoomMenu->setItemChecked(ID_ZOOM_BASE+zoom,TRUE);

  if(cubebox->maxZoom())
    {
      menu->setItemEnabled(ID_OPT_ZOOMIN,FALSE);
      toolbar->setItemEnabled(ID_OPT_ZOOMIN,FALSE);
    }
  else
    {
      menu->setItemEnabled(ID_OPT_ZOOMIN,TRUE);
      toolbar->setItemEnabled(ID_OPT_ZOOMIN,TRUE);
    }
  if(cubebox->minZoom())
    {
      menu->setItemEnabled(ID_OPT_ZOOMOUT,FALSE);
      toolbar->setItemEnabled(ID_OPT_ZOOMOUT,FALSE);
    }
  else
    {
      menu->setItemEnabled(ID_OPT_ZOOMOUT,TRUE);
      toolbar->setItemEnabled(ID_OPT_ZOOMOUT,TRUE);
    }

}	     
   

void TopWindow::changeColor(int player)
{
  QColor newColor;
  int result=KColorDialog::getColor(newColor);
  if(result)
    {
      if(player==1)
	{
	  delete color1;
	  color1=new QPalette(newColor);
	}    
      else if(player==2) 
	{
	  delete color2;
	  color2=new QPalette(newColor);
	}
      cubebox->setColors(color1,color2);
    }
} 


void TopWindow::saveSettings()
{
  KConfig *config=kapp->getConfig();
  bool status=options->isItemChecked(ID_VIEW_TOOLBAR);
  config->writeEntry("Toolbar",(int)(status));
  status=options->isItemChecked(ID_VIEW_STATUSBAR);
  config->writeEntry("Statusbar",(int)(status));
  config->writeEntry("Zoom",cubebox->zoom());
  config->writeEntry("CubeDim",cubebox->dim());
  config->writeEntry("Color1",color1->normal().background());
  config->writeEntry("Color2",color2->normal().background());
  config->writeEntry("Skill",cubebox->skill());
  config->writeEntry("Computer Pl.1",
		  (int)game->isItemChecked(ID_COMPUTER_BASE+1));
  config->writeEntry("Computer Pl.2",
		  (int)game->isItemChecked(ID_COMPUTER_BASE+2));
  
  config->sync();
}

 
// SLOT for handling menubar- and toolbarevents

void TopWindow::menuCallback(int item)
{
  // what is pressed

  switch(item)
    {
    case ID_GAME_QUIT:
       {
	cubebox->stopActivities();
	kapp->quit();
	break;
      }
    case ID_GAME_NEW:
      game->setItemEnabled(ID_GAME_UNDO,FALSE);
      toolbar->setItemEnabled(ID_GAME_UNDO,FALSE);
      cubebox->reset();
      break;
      
    case ID_GAME_OPEN:
      {
	KConfig *config=kapp->getConfig();
	KConfigGroupSaver groupSaver(config,QString("SavedGame"));
	int newDim=config->readNumEntry("CubeDim");
	if(newDim==0) // if no game was saved before
	  {
	    QString s(klocale->translate("There is no saved Game !"));
	    KMsgBox::message( 0, klocale->translate("Information"), 
			      (const char *)s,
			      KMsgBox::INFORMATION, klocale->translate("OK") );
	  }
	else
	  {
	     
	     if(cubebox->isActive())
		cubebox->stopActivities();
	
	     if(cubebox->dim() != newDim)
	       {
		 int zoom=cubebox->zoom();
		 int skill=cubebox->skill();
		 updatePlayfieldMenu(newDim);
		 // create new cubebox
		 delete cubebox;
		 createNewCubebox(newDim,zoom,skill);
		 
	         if(game->isItemChecked(ID_COMPUTER_BASE+1))
		    cubebox->setComputerplayer(1,TRUE);
		 if(game->isItemChecked(ID_COMPUTER_BASE+2))
		    cubebox->setComputerplayer(2,TRUE);
	       }
	    cubebox->restoreGame(config);
	    
	  }
	
	 game->setItemEnabled(ID_GAME_UNDO,FALSE);
	 toolbar->setItemEnabled(ID_GAME_UNDO,FALSE);
	
	break;
      }
     

    case ID_GAME_SAVE:
      {
	KConfig *config=kapp->getConfig();
	KConfigGroupSaver groupSaver(config,QString("SavedGame"));
	config->writeEntry("CubeDim",cubebox->dim());
	cubebox->saveGame(kapp->getConfig());
	QString s(klocale->translate("Game saved !"));
	KMsgBox::message( 0, klocale->translate("Information"), 
			  (const char *)s,
			  KMsgBox::INFORMATION, klocale->translate("OK") );
	break;
      }

   case ID_GAME_HINT:
      {
	 debug("Get Hint");
	 enableStop();
	 cubebox->getHint();
	 break;
      }
   case ID_GAME_STOP_HINT:
      {
	 disableStop();
	 debug("thinking stopped");
	 cubebox->stopActivities();
	 break;
      }
   case ID_COMPUTER_BASE+1:
   case ID_COMPUTER_BASE+2:
      {
	 bool flag=game->isItemChecked(item);
	 flag= flag ? FALSE : TRUE;
	 game->setItemChecked(item,flag);
	 cubebox->setComputerplayer(item-ID_COMPUTER_BASE,flag);
	 if(flag)
	    cubebox->checkComputerplayer(item-ID_COMPUTER_BASE);
	 break;
      }
   case ID_GAME_UNDO:
      {
	 if(!cubebox->undo())
	    return;
	 game->setItemEnabled(ID_GAME_UNDO,FALSE);
	 toolbar->setItemEnabled(ID_GAME_UNDO,FALSE);
	 break;
      }   
   case ID_SKILL_BASE+1: 
   case ID_SKILL_BASE+2: 
   case ID_SKILL_BASE+3: 
   case ID_SKILL_BASE+4: 
   case ID_SKILL_BASE+5:
      {
	 int oldSkill=cubebox->setSkill(item-ID_SKILL_BASE);
	 skillMenu->setItemChecked(ID_SKILL_BASE+oldSkill,FALSE);
	 skillMenu->setItemChecked(item,TRUE);	
	 break;
      }
      
            
   case ID_VIEW_TOOLBAR:
      {
	enableToolBar(); // toggles toolbar
	bool status=options->isItemChecked(ID_VIEW_TOOLBAR);
	options->setItemChecked(ID_VIEW_TOOLBAR, status?FALSE:TRUE);
	break;
      }
    case ID_VIEW_STATUSBAR:
      {
	enableStatusBar(); // toggles statusbar
	bool status=options->isItemChecked(ID_VIEW_STATUSBAR);
	options->setItemChecked(ID_VIEW_STATUSBAR, status?FALSE:TRUE);
	break;
      }

      // optionsMenu
    case ID_OPT_ZOOMIN:
      {
	updateZoomMenu(cubebox->zoomIn());
	updateRects();   // fit topwindow to new size 
	break;
      }
    
    case ID_OPT_ZOOMOUT:
      {
	updateZoomMenu(cubebox->zoomOut());
	updateRects();   // fit topwindow to new size
	break;
      }

    case ID_ZOOM_BASE+3:
    case ID_ZOOM_BASE+4:
    case ID_ZOOM_BASE+5:
    case ID_ZOOM_BASE+6:
    case ID_ZOOM_BASE+7:
    case ID_ZOOM_BASE+8:
      {
	cubebox->setZoom(item-ID_ZOOM_BASE);
	updateZoomMenu(item-ID_ZOOM_BASE);
	updateRects();
	break;
      }
   
/* 

    case ID_OPT_DEFAULT:
      {
	int cubeDim,cubeSize;
	setDefaultValues(cubeDim,cubeSize);
	if(cubebox)
	  delete cubebox;
	createNewCubebox(cubeDim,cubeSize);
	updatePlayfieldMenu(cubeDim);
	menu->setItemEnabled(ID_OPT_ZOOMIN,TRUE);
	menu->setItemEnabled(ID_OPT_ZOOMOUT,TRUE);
	toolbar->setItemEnabled(ID_OPT_ZOOMIN,TRUE);
	toolbar->setItemEnabled(ID_OPT_ZOOMOUT,TRUE);
	game->setItemEnabled(ID_GAME_UNDO,FALSE);
	toolbar->setItemEnabled(ID_GAME_UNDO,FALSE);
	cubebox->reset();
	break;
      }
 */
    case ID_OPT_SAVE:
      saveSettings();
      break;

    case ID_COLOR_BASE+1:
    case ID_COLOR_BASE+2:
      changeColor(item-ID_COLOR_BASE);
      break;

    // set new cubedim
	      
    case (ID_FIELD+5):
    case (ID_FIELD+6):
    case (ID_FIELD+7):
    case (ID_FIELD+8):
    case (ID_FIELD+9):
    case (ID_FIELD+10):
       {
	 if(cubebox->isActive())
		 break;
	  
	 if(cubebox->dim() != item-ID_FIELD)
	   {     
	     int cubeDim=item-ID_FIELD;
	     int zoom=cubebox->zoom();
	     int skill=cubebox->skill();
	     updatePlayfieldMenu(cubeDim);
	     // create new cubebox
	     delete cubebox;
	     createNewCubebox(cubeDim,zoom,skill);
	     
	     cubebox->reset();
	     
	     if(game->isItemChecked(ID_COMPUTER_BASE+1))
		cubebox->setComputerplayer(1,TRUE);
	     if(game->isItemChecked(ID_COMPUTER_BASE+2))
		cubebox->setComputerplayer(2,TRUE);
	 
	     game->setItemEnabled(ID_GAME_UNDO,FALSE);
	     toolbar->setItemEnabled(ID_GAME_UNDO,FALSE);
	     
	     cubebox->checkComputerplayer(1);
	   }
	 break;
      }

      // About
    case ID_HELP_ABOUT:
      {
	QString s;
	s.sprintf(klocale->translate("JumpingCube Version %s \n\n (C) 1998 by Matthias Kiefer \nMatthias.Kiefer@stud.uni-karlsruhe.de\n\n"),VERSION);
	s+=klocale->translate("This program is free software; you can redistribute it\n");
	s+=klocale->translate("and/or modify it under the terms of the GNU General\n");
	s+=klocale->translate("Public License as published by the Free Software\n");
	s+=klocale->translate("Foundation; either version 2 of the License, or\n");
	s+=klocale->translate("(at your option) any later version.");
	KMsgBox::message( 0, klocale->translate("About JumpingCube"), 
		       (const char *)s,
		       KMsgBox::INFORMATION, klocale->translate("OK") );
	break;
      }
   
    case ID_HELP_CONTENTS:
      kapp->invokeHTMLHelp("","");
      break;
      
      

    } // end switch

}

void TopWindow::changePlayer(int newPlayer)
{
  QString s;
  s.sprintf(klocale->translate("on turn: Player %d"),newPlayer);
  if(cubebox->isComputer(newPlayer))
     s+=klocale->translate("(Computer)");
  
  statusbar->changeItem(s,ID_STATUS_TURN);

  game->setItemEnabled(ID_GAME_UNDO,TRUE);
  toolbar->setItemEnabled(ID_GAME_UNDO,TRUE);
}


void TopWindow::barPositionChanged()
{
  KConfig *config= kapp->getConfig();
  if(config != 0)
    {
      config->writeEntry("MenubarPos",(int)(menu->menuBarPos()));
      config->writeEntry("ToolbarPos",(int)(toolbar->barPos()));
    }
}


void TopWindow::disableStop()
{
   toolbar->setItemEnabled(ID_GAME_STOP_HINT,FALSE);
   game->setItemEnabled(ID_GAME_STOP_HINT,FALSE);
}

void TopWindow::enableStop()
{
   toolbar->setItemEnabled(ID_GAME_STOP_HINT,TRUE);
   game->setItemEnabled(ID_GAME_STOP_HINT,TRUE);
}

void TopWindow::saveProperties(KConfig *config)
{  
  // just for the menuentries
  bool status=options->isItemChecked(ID_VIEW_TOOLBAR);
  config->writeEntry("Toolbar",(int)(status));
  status=options->isItemChecked(ID_VIEW_STATUSBAR);
  config->writeEntry("Statusbar",(int)(status));
  
  config->writeEntry("Zoom",cubebox->zoom());
  config->writeEntry("CubeDim",cubebox->dim());
  config->writeEntry("Color1",color1->normal().background());
  config->writeEntry("Color2",color2->normal().background());
  config->writeEntry("Skill",cubebox->skill());
  config->writeEntry("Computer Pl.1",
		  (int)game->isItemChecked(ID_COMPUTER_BASE+1));
  config->writeEntry("Computer Pl.2",
		  (int)game->isItemChecked(ID_COMPUTER_BASE+2));
  
  cubebox->saveGame(config);

}


void TopWindow::readProperties(KConfig *config)
{
  int i;
  bool visible=(bool)(config->readNumEntry("Toolbar",(int)(TRUE)));
  options->setItemChecked(ID_VIEW_TOOLBAR,visible);
  visible=(bool)(config->readNumEntry("Statusbar",(int)(TRUE)));
  options->setItemChecked(ID_VIEW_STATUSBAR,visible);
  
  int zoom=config->readNumEntry("Zoom",DEFAULT_ZOOM);
  int cubeDim=config->readNumEntry("CubeDim",DEFAULT_DIM);
  color1=new QPalette(config->readColorEntry("Color1",&QColor("darkred")));
  color2=new QPalette(config->readColorEntry("Color2",&QColor("darkblue")));
  

  createNewCubebox(cubeDim,zoom);

  updatePlayfieldMenu(cubeDim);
  updateZoomMenu(cubebox->zoom());
  
  int skill=config->readNumEntry("Skill",1);
  cubebox->setSkill(skill);
  for(i=1;i<6;i++)
  {
     skillMenu->setItemChecked(ID_SKILL_BASE+i,FALSE);
  }
  skillMenu->setItemChecked(ID_SKILL_BASE+skill,TRUE);
   
  bool flag=(bool)config->readNumEntry("Computer Pl.1",(int)FALSE);
  cubebox->setComputerplayer(1,flag);
  game->setItemChecked(ID_COMPUTER_BASE+1,flag);
  flag=(bool)config->readNumEntry("Computer Pl.2",(int)TRUE);
  cubebox->setComputerplayer(2,flag);
  game->setItemChecked(ID_COMPUTER_BASE+2,flag);

  cubebox->restoreGame(config);
  
  game->setItemEnabled(ID_GAME_UNDO,FALSE);
  toolbar->setItemEnabled(ID_GAME_UNDO,FALSE);
}


void TopWindow::closeEvent(QCloseEvent *e)
{
   cubebox->stopActivities();
   e->accept();
}


//#include "topwindow.moc"

